// This file is generated by kconfig_compiler_kf6 from phpdocssettings.kcfg.
// All changes you do to this file will be lost.
#ifndef PHPDOCSSETTINGS_H
#define PHPDOCSSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class PhpDocsSettings : public KConfigSkeleton
{
  public:

    static PhpDocsSettings *self();
    ~PhpDocsSettings() override;

    /**
      Get Specifies the location of the PHP documentation to use.
    */
    static
    QUrl phpDocLocation()
    {
      return self()->mPhpDocLocation;
    }

    /**
      Is Specifies the location of the PHP documentation to use. Immutable
    */
    static
    bool isPhpDocLocationImmutable()
    {
      return self()->isImmutable( QStringLiteral( "phpDocLocation" ) );
    }

    /**
      Get Item object corresponding to phpDocLocation()
    */
    ItemUrl *phpDocLocationItem()
    {
      return mPhpDocLocationItem;
    }

  protected:
    PhpDocsSettings();
    friend class PhpDocsSettingsHelper;


    // PHP Documentation
    QUrl mPhpDocLocation;

  private:
    ItemUrl *mPhpDocLocationItem;
};

#endif

