/********************************************************************************
** Form generated from reading UI file 'phpdocsconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PHPDOCSCONFIG_H
#define UI_PHPDOCSCONFIG_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_PhpDocsConfigUI
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *PhpDoc;
    QVBoxLayout *verticalLayout;
    QLabel *PhpDocNote;
    QFormLayout *formLayout;
    QLabel *PhpDocLocationLabel;
    KUrlRequester *kcfg_phpDocLocation;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *PhpDocsConfigUI)
    {
        if (PhpDocsConfigUI->objectName().isEmpty())
            PhpDocsConfigUI->setObjectName("PhpDocsConfigUI");
        PhpDocsConfigUI->resize(560, 404);
        PhpDocsConfigUI->setLocale(QLocale(QLocale::English, QLocale::UnitedStates));
        verticalLayout_2 = new QVBoxLayout(PhpDocsConfigUI);
        verticalLayout_2->setObjectName("verticalLayout_2");
        PhpDoc = new QGroupBox(PhpDocsConfigUI);
        PhpDoc->setObjectName("PhpDoc");
        PhpDoc->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        verticalLayout = new QVBoxLayout(PhpDoc);
        verticalLayout->setObjectName("verticalLayout");
        PhpDocNote = new QLabel(PhpDoc);
        PhpDocNote->setObjectName("PhpDocNote");
        PhpDocNote->setLocale(QLocale(QLocale::English, QLocale::UnitedStates));
        PhpDocNote->setScaledContents(false);
        PhpDocNote->setWordWrap(true);
        PhpDocNote->setOpenExternalLinks(true);

        verticalLayout->addWidget(PhpDocNote);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        PhpDocLocationLabel = new QLabel(PhpDoc);
        PhpDocLocationLabel->setObjectName("PhpDocLocationLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, PhpDocLocationLabel);

        kcfg_phpDocLocation = new KUrlRequester(PhpDoc);
        kcfg_phpDocLocation->setObjectName("kcfg_phpDocLocation");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_phpDocLocation);


        verticalLayout->addLayout(formLayout);


        verticalLayout_2->addWidget(PhpDoc);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(PhpDocsConfigUI);

        QMetaObject::connectSlotsByName(PhpDocsConfigUI);
    } // setupUi

    void retranslateUi(QWidget *PhpDocsConfigUI)
    {
        PhpDoc->setTitle(tr2i18n("PHP Manual Integration", nullptr));
        PhpDocNote->setText(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">By default the online <a href=\"https://php.net\"><span style=\" text-decoration: underline; color:#0000ff;\">PHP documentation</span></a> is used remotely. Alternatively you can set the location to a local copy of the documentation in the language of your choice. Make sure you downloaded the <span style=\" font-style:italic;\">\"Many HTML files\"</span> package.</p></body></html>", nullptr));
        PhpDocLocationLabel->setText(tr2i18n("Location:", nullptr));
        (void)PhpDocsConfigUI;
    } // retranslateUi

};

namespace Ui {
    class PhpDocsConfigUI: public Ui_PhpDocsConfigUI {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PHPDOCSCONFIG_H

