// THIS FILE IS GENERATED BY kdev-pg-qt 2.4.0
// WARNING! All changes made in this file will be lost!

#ifndef PHP_DEFAULT_VISITOR_H_INCLUDED
#define PHP_DEFAULT_VISITOR_H_INCLUDED

#include "phpvisitor.h"

#include "parserexport.h"
namespace Php
{

class KDEVPHPPARSER_EXPORT DefaultVisitor: public Visitor
{
public:
    void visitAdditiveExpression(AdditiveExpressionAst *node) override;
    void visitAdditiveExpressionRest(AdditiveExpressionRestAst *node) override;
    void visitArrayIndexSpecifier(ArrayIndexSpecifierAst *node) override;
    void visitArrayPairValue(ArrayPairValueAst *node) override;
    void visitAssignmentExpression(AssignmentExpressionAst *node) override;
    void visitAssignmentExpressionCheckIfVariable(AssignmentExpressionCheckIfVariableAst *node) override;
    void visitAssignmentExpressionEqual(AssignmentExpressionEqualAst *node) override;
    void visitAssignmentList(AssignmentListAst *node) override;
    void visitAssignmentListElement(AssignmentListElementAst *node) override;
    void visitBaseVariable(BaseVariableAst *node) override;
    void visitBaseVariableWithFunctionCalls(BaseVariableWithFunctionCallsAst *node) override;
    void visitBitAndExpression(BitAndExpressionAst *node) override;
    void visitBitOrExpression(BitOrExpressionAst *node) override;
    void visitBitXorExpression(BitXorExpressionAst *node) override;
    void visitBooleanAndExpression(BooleanAndExpressionAst *node) override;
    void visitBooleanOrExpression(BooleanOrExpressionAst *node) override;
    void visitCaseList(CaseListAst *node) override;
    void visitCase_item(Case_itemAst *node) override;
    void visitCatchItem(CatchItemAst *node) override;
    void visitClassBody(ClassBodyAst *node) override;
    void visitClassConstant(ClassConstantAst *node) override;
    void visitClassConstantDeclaration(ClassConstantDeclarationAst *node) override;
    void visitClassDeclarationStatement(ClassDeclarationStatementAst *node) override;
    void visitClassExtends(ClassExtendsAst *node) override;
    void visitClassImplements(ClassImplementsAst *node) override;
    void visitClassName(ClassNameAst *node) override;
    void visitClassNameReference(ClassNameReferenceAst *node) override;
    void visitClassProperty(ClassPropertyAst *node) override;
    void visitClassStatement(ClassStatementAst *node) override;
    void visitClassVariable(ClassVariableAst *node) override;
    void visitClassVariableDeclaration(ClassVariableDeclarationAst *node) override;
    void visitClosure(ClosureAst *node) override;
    void visitCommonScalar(CommonScalarAst *node) override;
    void visitCompoundNamespace(CompoundNamespaceAst *node) override;
    void visitCompoundNamespaceStatement(CompoundNamespaceStatementAst *node) override;
    void visitCompoundVariable(CompoundVariableAst *node) override;
    void visitCompoundVariableWithSimpleIndirectReference(CompoundVariableWithSimpleIndirectReferenceAst *node) override;
    void visitConditionalExpression(ConditionalExpressionAst *node) override;
    void visitConstantDeclaration(ConstantDeclarationAst *node) override;
    void visitConstantOrClassConst(ConstantOrClassConstAst *node) override;
    void visitCtorArguments(CtorArgumentsAst *node) override;
    void visitDeclareItem(DeclareItemAst *node) override;
    void visitDeclareStatement(DeclareStatementAst *node) override;
    void visitDimListItem(DimListItemAst *node) override;
    void visitElseSingle(ElseSingleAst *node) override;
    void visitElseifList(ElseifListAst *node) override;
    void visitElseifListItem(ElseifListItemAst *node) override;
    void visitEncaps(EncapsAst *node) override;
    void visitEncapsList(EncapsListAst *node) override;
    void visitEncapsVar(EncapsVarAst *node) override;
    void visitEncapsVarOffset(EncapsVarOffsetAst *node) override;
    void visitEqualityExpression(EqualityExpressionAst *node) override;
    void visitEqualityExpressionRest(EqualityExpressionRestAst *node) override;
    void visitExpr(ExprAst *node) override;
    void visitForExpr(ForExprAst *node) override;
    void visitForStatement(ForStatementAst *node) override;
    void visitForeachStatement(ForeachStatementAst *node) override;
    void visitForeachVariable(ForeachVariableAst *node) override;
    void visitFunctionCall(FunctionCallAst *node) override;
    void visitFunctionCallParameterList(FunctionCallParameterListAst *node) override;
    void visitFunctionCallParameterListElement(FunctionCallParameterListElementAst *node) override;
    void visitFunctionDeclarationStatement(FunctionDeclarationStatementAst *node) override;
    void visitGenericTypeHint(GenericTypeHintAst *node) override;
    void visitGlobalVar(GlobalVarAst *node) override;
    void visitIdentifier(IdentifierAst *node) override;
    void visitIifeSyntax(IifeSyntaxAst *node) override;
    void visitInnerStatementList(InnerStatementListAst *node) override;
    void visitInnerUseNamespace(InnerUseNamespaceAst *node) override;
    void visitInstantiationAccess(InstantiationAccessAst *node) override;
    void visitInterfaceDeclarationStatement(InterfaceDeclarationStatementAst *node) override;
    void visitLexicalVar(LexicalVarAst *node) override;
    void visitLexicalVarList(LexicalVarListAst *node) override;
    void visitLogicalAndExpression(LogicalAndExpressionAst *node) override;
    void visitLogicalOrExpression(LogicalOrExpressionAst *node) override;
    void visitLogicalXorExpression(LogicalXorExpressionAst *node) override;
    void visitMethodBody(MethodBodyAst *node) override;
    void visitMultiplicativeExpression(MultiplicativeExpressionAst *node) override;
    void visitMultiplicativeExpressionRest(MultiplicativeExpressionRestAst *node) override;
    void visitNamespaceDeclarationStatement(NamespaceDeclarationStatementAst *node) override;
    void visitNamespacedIdentifier(NamespacedIdentifierAst *node) override;
    void visitNamespacedIdentifierBeforeGroupedNamespace(NamespacedIdentifierBeforeGroupedNamespaceAst *node) override;
    void visitNewElseSingle(NewElseSingleAst *node) override;
    void visitNewElseifList(NewElseifListAst *node) override;
    void visitNewelseifListItem(NewelseifListItemAst *node) override;
    void visitNullCoalesceExpression(NullCoalesceExpressionAst *node) override;
    void visitObjectDimList(ObjectDimListAst *node) override;
    void visitObjectOperator(ObjectOperatorAst *node) override;
    void visitObjectProperty(ObjectPropertyAst *node) override;
    void visitOptionalClassModifier(OptionalClassModifierAst *node) override;
    void visitOptionalModifiers(OptionalModifiersAst *node) override;
    void visitOuterTopStatement(OuterTopStatementAst *node) override;
    void visitParameter(ParameterAst *node) override;
    void visitParameterList(ParameterListAst *node) override;
    void visitParameterType(ParameterTypeAst *node) override;
    void visitParameterTypeHint(ParameterTypeHintAst *node) override;
    void visitPostprefixOperator(PostprefixOperatorAst *node) override;
    void visitPrintExpression(PrintExpressionAst *node) override;
    void visitPropertyType(PropertyTypeAst *node) override;
    void visitPropertyTypeHint(PropertyTypeHintAst *node) override;
    void visitRelationalExpression(RelationalExpressionAst *node) override;
    void visitRelationalExpressionRest(RelationalExpressionRestAst *node) override;
    void visitReservedNonModifierIdentifier(ReservedNonModifierIdentifierAst *node) override;
    void visitReservedNonModifiers(ReservedNonModifiersAst *node) override;
    void visitReturnType(ReturnTypeAst *node) override;
    void visitReturnTypeHint(ReturnTypeHintAst *node) override;
    void visitScalar(ScalarAst *node) override;
    void visitSemiReserved(SemiReservedAst *node) override;
    void visitSemiReservedIdentifier(SemiReservedIdentifierAst *node) override;
    void visitSemicolonOrCloseTag(SemicolonOrCloseTagAst *node) override;
    void visitShiftExpression(ShiftExpressionAst *node) override;
    void visitShiftExpressionRest(ShiftExpressionRestAst *node) override;
    void visitStart(StartAst *node) override;
    void visitStatement(StatementAst *node) override;
    void visitStaticArrayPairValue(StaticArrayPairValueAst *node) override;
    void visitStaticMember(StaticMemberAst *node) override;
    void visitStaticProperty(StaticPropertyAst *node) override;
    void visitStaticScalar(StaticScalarAst *node) override;
    void visitStaticVar(StaticVarAst *node) override;
    void visitStringIndexSpecifier(StringIndexSpecifierAst *node) override;
    void visitSwitchCaseList(SwitchCaseListAst *node) override;
    void visitTopStatement(TopStatementAst *node) override;
    void visitTraitAliasDeclaration(TraitAliasDeclarationAst *node) override;
    void visitTraitAliasIdentifier(TraitAliasIdentifierAst *node) override;
    void visitTraitAliasStatement(TraitAliasStatementAst *node) override;
    void visitTraitDeclarationStatement(TraitDeclarationStatementAst *node) override;
    void visitTraitVisibilityModifiers(TraitVisibilityModifiersAst *node) override;
    void visitUnaryExpression(UnaryExpressionAst *node) override;
    void visitUnaryExpressionNotPlusminus(UnaryExpressionNotPlusminusAst *node) override;
    void visitUnionParameterType(UnionParameterTypeAst *node) override;
    void visitUnionPropertyType(UnionPropertyTypeAst *node) override;
    void visitUnionReturnType(UnionReturnTypeAst *node) override;
    void visitUseNamespaceOrUseGroupedNamespace(UseNamespaceOrUseGroupedNamespaceAst *node) override;
    void visitUseStatement(UseStatementAst *node) override;
    void visitVarExpression(VarExpressionAst *node) override;
    void visitVarExpressionArray(VarExpressionArrayAst *node) override;
    void visitVarExpressionNewObject(VarExpressionNewObjectAst *node) override;
    void visitVarExpressionNormal(VarExpressionNormalAst *node) override;
    void visitVariable(VariableAst *node) override;
    void visitVariableIdentifier(VariableIdentifierAst *node) override;
    void visitVariableName(VariableNameAst *node) override;
    void visitVariableObjectProperty(VariableObjectPropertyAst *node) override;
    void visitVariableProperty(VariablePropertyAst *node) override;
    void visitVariableWithoutObjects(VariableWithoutObjectsAst *node) override;
    void visitWhileStatement(WhileStatementAst *node) override;
};

} // end of namespace Php

#endif

